# Interface implementation Ball And Beam ====================================================

# There is a union type defined for convenience:
# AbstractETHHelicopter = Union{ETHHelicopter, ETHHelicopterSimulator}
# Although not Abstract per se, the names AbstractETHHelicopter etc. were chosen since this
# reflects the usage in dispatch.

export ETHHelicopter, ETHHelicopterSimulator, AbstractETHHelicopter

struct ETHHelicopter <: PhysicalProcess
	h::Float64
    bias::Float64
    stream::LabStream
    measure1::AnalogInput10V
    measure2::AnalogInput10V
    control1::AnalogOutput10V
    control2::AnalogOutput10V
end
ETHHelicopter() = ETHHelicopter(0.050, 0., ComediStream(), AnalogInput10V(0), AnalogInput10V(1), AnalogOutput10V(0), AnalogOutput10V(1))

struct ETHHelicopterSimulator <: SimulatedProcess
    h::Float64
    bias::Float64
    state::Vector{Float64}
end
ETHHelicopterSimulator() = ETHHelicopterSimulator(0.01, zeros(4))

const AbstractETHHelicopter = Union{ETHHelicopter, ETHHelicopterSimulator}
num_outputs(p::AbstractETHHelicopter) = 2
num_inputs(p::AbstractETHHelicopter)  = 2
outputrange(p::AbstractETHHelicopter) = [(-10,10),(-10,10)]
inputrange(p::AbstractETHHelicopter)  = [(-10,10),(-10,10)]
isstable(p::AbstractETHHelicopter)    = false
isasstable(p::AbstractETHHelicopter)  = false
sampletime(p::AbstractETHHelicopter)  = p.h
bias(p::AbstractETHHelicopter)        = p.bias


function control(p::ETHHelicopter, u)
    send(p.control1,u[1])
    send(p.control2,u[2])
end

measure(p::ETHHelicopter) = [read(p.measure1), read(p.measure2)] #i=0 for pitch, i=1 for yaw


control(p::ETHHelicopterSimulator, u)  = error("Not yet implemented")
measure(p::ETHHelicopterSimulator)     = error("Not yet implemented")



initialize(p::ETHHelicopter) = init_devices!(p.stream, p.measure1, p.measure2, p.control1, p.control2)
finalize(p::ETHHelicopter)   = close(p.stream)
initialize(p::ETHHelicopterSimulator) = nothing
finalize(p::ETHHelicopterSimulator)   = nothing
